/*
 BibbleBomb for MIB / Tsumutenkaku
 May 21, 2015
 Tandem Systems, Inc.
 Masaru Hashimoto
 
 Ttsumutenkaku Version May 23-24, 2015
 Maker's Bazaar Version June 27-28, 2015
   http://makersbazaar.jp
 
 Libraries
 https://github.com/sparkfun/SevSeg
 http://playground.arduino.cc/Code/Bounce
 
 */

#include <SevSeg.h>
#include <Bounce2.h>

SevSeg myDisplay;

/* 動作フラグ */
#define COUNT_DOWN 0 // カウントダウンフラグ
#define INIT_ERROR 1 // 初期エラーフラグ
#define TIMEUP 2 // タイムアップフラグ
#define CLEARED 3 // 解除完了フラグ
int bombState = COUNT_DOWN;

/* タイマー関係 */
unsigned long timer;
int deciSecond = 0; // timer の現在値、最初に startDeciSecond で初期化され 0 でタイムアップ、単位は 1/10 秒
const int startDeciSecond = 1800; // タイマーの初期値、単位は 1/10 秒、1800 は 180 秒（3分）
const int timerUpdateMills = 100; // 7seg LED の表示更新間隔、単位は 1/100 秒、100 で 0.1 秒毎に表示を更新
const int subtractDeciSecondNormal = 1; // タイマーの表示更新間隔毎の減算値、正常値、単位は 1/10 秒
const int subtractDeciSecondHurry = 5; // タイマーの表示更新間隔毎の減算値、急速版、単位は 1/10 秒
int subtractDeciSecond = 1; // タイマーの表示更新間隔毎の減算値、1 で時間通り、単位は 1/10 秒

/* ピンアサイン */
#define CANCEL_PINS_NUM 5 // ばくだん解除コード数

const int vibPin = 0; // D0 振動モーター
const int ledPin = 1; // D1 成功時のグリーンLED
const int cancelPins[] = {14,15,16,17,18}; // A0-A4 ばくだん解除コードピン
const int bombTypePin = 19; // A5 ばくだん種別 1号はオン、２号はオフ

/* ばくだん解除コードのチャタリング防止用 */
Bounce bouncers[CANCEL_PINS_NUM]; // setup で初期化
#define BOUNCE_INTERVAL 500 // チャタリング不検知時間、ミリ秒

/* INPUT_PULLUP 用の on/off 値 */

const int buttonON = LOW; // GND に繋がっているとピンの値はLOW
const int buttonOFF = HIGH;

/* ばくだん解除コードの遷移 */
#define CANCEL_STEPS_NUM 7 // 解除ステップ数

const byte cancelOrders1[] = {31,27,11,3,1,17,16}; // 1号
const byte cancelOrders2[] = {31,15,7,6,2,10,8}; // 2号
byte cancelOrders[CANCEL_STEPS_NUM];
int currentStep = 0; // 解除ステップインデックス

/* ばくだん解除コード状態監視値  */
int lastPinValue; // 前回値 setup で初期化

/* 7seg桁数*/
#define NUMBER_OF_DIGITS 4

/* ばくだん解除コード状態の取得 */

int getPinValue() {
  int pinValue = 0;
  for (int i=0; i < CANCEL_PINS_NUM; i++) {
    if ( bouncers[i].read() == buttonON ) {
      bitSet(pinValue, i);
    }
  }
  return pinValue;
}


void setup()
{ 
  
  /* 入出力の初期化 */
  
  /* 振動モーター D0 */
  pinMode(vibPin, OUTPUT); 
  digitalWrite(vibPin, LOW);
  
  /* 成功時のグリーンLED D1 */
  pinMode(ledPin, OUTPUT);
  digitalWrite(ledPin, LOW);
  
  /* ばくだん解除コード */
  for (int i=0; i < CANCEL_PINS_NUM; i++) {
    bouncers[i] = Bounce();
    bouncers[i].interval(BOUNCE_INTERVAL);
    pinMode(cancelPins[i], INPUT_PULLUP);
    bouncers[i].attach( cancelPins[i] );
  }
  
  /* ばくだん種別 */
  pinMode(bombTypePin, INPUT_PULLUP); 
  
  /* 7seg bubble dispaly の設定*/
  // Declare what pins are connected to the GND pins (cathodes)
  int digit1 = 8; //Pin 1
  int digit2 = 5; //Pin 10
  int digit3 = 11; //Pin 4
  int digit4 = 13; //Pin 6
  // Declare what pins are connected to the segments (anodes)
  int segA = 7; //Pin 12
  int segB = 6; //Pin 11
  int segC = 10; //Pin 3
  int segD = 3; //Pin 8
  int segE = 9; //Pin 2
  int segF = 4; //Pin 9
  int segG = 2; //Pin 7
  int segDP= 12; //Pin 5
  int displayType = COMMON_CATHODE; // Your display is either common cathode or common anode
  
  myDisplay.Begin(displayType, NUMBER_OF_DIGITS, digit1, digit2, digit3, digit4, segA, segB, segC, segD, segE, segF, segG, segDP);
  myDisplay.SetBrightness(100); //Set the display to 100% brightness level

  // ばくだん1,2号の設定
  if ( digitalRead(bombTypePin) == buttonON ) {
    memcpy( cancelOrders, cancelOrders1, CANCEL_STEPS_NUM );
  } else {
    memcpy( cancelOrders, cancelOrders2, CANCEL_STEPS_NUM );
  }
  lastPinValue = cancelOrders[0];
  
  // ばくだん解除コードの初期エラーチェック
  if ( getPinValue() != cancelOrders[0] ) {
    bombState = INIT_ERROR;
  }
  
  // カウントダウンタイマーの設定
  deciSecond = startDeciSecond;

  /* タイマーの初期値設定 */
  timer = millis();
  
}

void loop()
{
  
  if ( bombState == COUNT_DOWN ) {
    
    // 0.1 秒毎にタイムアップを確認
    if ( millis() - timer >= timerUpdateMills ) {
      timer = millis();
      if ( bombState == COUNT_DOWN ) deciSecond = deciSecond - subtractDeciSecond;
      if ( deciSecond <= 0 ) {
        bombState = TIMEUP;
      }
    }
      
    char displayStrings[NUMBER_OF_DIGITS]; // sprintf で利用
    sprintf(displayStrings, "%4d", deciSecond); //Convert deciSecond into a string that is right adjusted
    myDisplay.DisplayString(displayStrings, NUMBER_OF_DIGITS);
    digitalWrite(vibPin, LOW);
    digitalWrite(ledPin, LOW);
    
    int totalPinValue = lastPinValue; // 現在値

    int bounces = 0;
    for (int i=0; i < CANCEL_PINS_NUM; i++) {
      bounces = bounces + bouncers[i].update();
    }
    if ( bounces != 0 ) {
      // ピンが更新された
      int totalPinValue = getPinValue();
      if ( totalPinValue != lastPinValue ) {
        // ピンの状態が推移した
        if ( cancelOrders[currentStep + 1] == totalPinValue ) {
          lastPinValue = totalPinValue;
          currentStep++;
          subtractDeciSecond = subtractDeciSecondNormal;
          digitalWrite(ledPin, HIGH);
          delay(100);
          if ( currentStep == (CANCEL_STEPS_NUM - 1) ) {
            // クリアした
            bombState = CLEARED;
          }
        } else {
          subtractDeciSecond = subtractDeciSecondHurry;
          digitalWrite(vibPin, HIGH);
          delay(100);
        }
      }
    }
  
  }
  
  if ( bombState == INIT_ERROR ) {
    digitalWrite(ledPin, HIGH);
    myDisplay.DisplayString("ERR0", 0);
  }
  
  if ( bombState == TIMEUP ) {
    digitalWrite(vibPin, HIGH);
    myDisplay.DisplayString("DEAD", 0);
  }
  
  if ( bombState == CLEARED ) {
    digitalWrite(ledPin, HIGH);
    myDisplay.DisplayString("PASS", 0);
  }

  delay(5);
}

